<?php

declare(strict_types=1);

/**
 * MySQLデータベースをPDOで操作するための基本クラス
 * 
 * @author Yuuji Mimura
 */

namespace db;

use PDO;
use Throwable;

/**
 * PDOオブジェクトのシングルトンクラス定義
 */
class PDOSingleton
{
    private static PDO $singleton;
    private PDO $conn;

    /**
     * コンストラクタ
     */
    private function __construct($dsn, $username, $password)
    {
        $this->conn = new PDO($dsn, $username, $password);
        $this->conn->setAttribute(PDO::ATTR_DEFAULT_FETCH_MODE, PDO::FETCH_ASSOC);
        $this->conn->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
        $this->conn->setAttribute(PDO::ATTR_EMULATE_PREPARES, false);
    }

    /**
     * 単一のPDOオブジェクトを返します
     */
    public static function getInstance($dsn, $username, $password)
    {
        if (!isset(self::$singleton)) {
            $instance = new PDOSingleton($dsn, $username, $password);
            self::$singleton = $instance->conn;
        }
        return self::$singleton;
    }
}

/**
 * MySQL接続クラス定義
 */
class DataSource
{
    private PDO $conn;
    private $sqlResult;
    public const CLS = 'cls';

    /**
     * コンストラクタ
     */
    public function __construct($host = 'localhost', $port = MYSQL_PORT, $dbName = 'yelpcamp', $username = 'yelpcamp', $password = 'password')
    {
        try {
            $dsn = "mysql:host={$host};port={$port};dbname={$dbName};";
            $this->conn = PDOSingleton::getInstance($dsn, $username, $password);
        } catch (Throwable $e) {
            die('Datasource error.');
        }
    }

    public function select($sql = "", $params = [], $type = '', $cls = ''): array
    {
        $stmt = $this->executeSql($sql, $params);

        if ($type === static::CLS) {

            return $stmt->fetchAll(PDO::FETCH_CLASS, $cls);
        } else {

            return $stmt->fetchAll();
        }
    }

    public function execute($sql = "", $params = [])
    {
        $this->executeSql($sql, $params);
        return  $this->sqlResult;
    }

    public function selectOne($sql = "", $params = [], $type = '', $cls = '')
    {

        $result = $this->select($sql, $params, $type, $cls);
        return count($result) > 0 ? $result[0] : false;
    }

    public function begin()
    {
        $this->conn->beginTransaction();
    }

    public function commit()
    {
        $this->conn->commit();
    }

    public function rollback()
    {
        $this->conn->rollback();
    }

    private function executeSql($sql, $params)
    {
        $stmt = $this->conn->prepare($sql);
        $this->sqlResult = $stmt->execute($params);
        return $stmt;
    }

    public function lastInsertId()
    {
        return $this->conn->lastInsertId();
    }
}
