<?php

namespace models;

use models\ImageSchema;
use utils\YelpCampError;

class CampGroundSchema
{
    public int $id;
    public string $title;
    public int $price;
    public string $description;
    public string $location;
    public array $images;
    public string $author; // = users.username
    public string $geometry;
    public array $deleteImages;

    public static function getModel(): self
    {
        $params = get_param('campground', '');
        $title = get_item($params, 'title');
        $location = get_item($params, 'location');
        $price = get_item($params, 'price');
        $description = get_item($params, 'description');

        if (empty($title)) throw new YelpCampError('キャンプ名がありません。');
        if (empty($location)) throw new YelpCampError('住所がありません。');
        if (empty($price)) throw new YelpCampError('値段がありません。');
        if (!is_numeric($price)) throw new YelpCampError('値段が数値ではありません。');
        if (intval($price) < 0) throw new YelpCampError('値段がマイナスになっています。');
        if (empty($description)) throw new YelpCampError('説明が入力されていません。');

        $camp = new CampGroundSchema;
        $camp->title = $title;
        $camp->location = $location;
        $camp->price = intval($price);
        $camp->description = $description;
        $camp->images = self::getImages();

        $deleteImages = get_param('deleteImages', '');
        $camp->deleteImages = is_array($deleteImages) ? $deleteImages : [];

        return $camp;
    }

    public function isOwner(UserSchema|null $user): bool
    {
        return !empty($user) && $user->username === $this->author;
    }

    public function addImages(array $images)
    {
        if (empty($this->images)) $this->images = [];

        foreach ($images as $image) {
            $img = new ImageSchema;
            $img->filename = $image;

            $fname = pathinfo($image, PATHINFO_FILENAME);
            $ext = pathinfo($image, PATHINFO_EXTENSION);
            $thumb_name = self::getUniqueFname("../uploads/thumbnail/{$fname}.{$ext}") ?? '';

            $img->thumbnail = self::createThubnail("../{$image}", $thumb_name) ? "uploads/thumbnail/{$fname}.{$ext}" : $image;
            array_push($this->images, $img);
        }
    }

    public function getGeoData()
    {
        $href = BASE_CONTEXT_PATH . "campgrounds/{$this->id}";
        $summary = mb_substr($this->description, 0, 20);
        if (mb_strlen($this->description) > 20) $summary .= "...";
        $geo_info = [
            'geometry' => json_decode($this->geometry),
            'title' => $this->title,
            'location' => $this->location,
            'properties' => ['popupMarkup' => "<strong><a href=\"{$href}\">{$this->title}</a></strong>\n<p>{$summary}</p>"]
        ];

        return $geo_info;
    }

    private static function getUniqueFname($filename)
    {
        $dname = pathinfo($filename, PATHINFO_DIRNAME);
        $fname = pathinfo($filename, PATHINFO_FILENAME);
        $ext = pathinfo($filename, PATHINFO_EXTENSION);

        $counter = 1;
        $temp_fname = $fname;

        try {
            $mask = umask(002);
            if (!is_dir($dname) && !mkdir($dname, 0775, true)) return null;
        } finally {
            umask($mask);
        }

        while (file_exists("{$dname}/{$temp_fname}.{$ext}")) {
            $temp_fname = "{$fname}-{$counter}";
            $counter += 1;
        }

        return "{$dname}/{$temp_fname}.{$ext}";
    }

    private static function getImages()
    {
        $results = [];

        if (!isset($_FILES['image'])) return [];

        foreach ($_FILES['image']['error'] as $key => $error) {
            if ($error) continue;
            $fname = basename($_FILES['image']['name'][$key]);
            $save_name = self::getUniqueFname('uploads/' . $fname);
            if (empty($save_name)) continue;
            if (!move_uploaded_file($_FILES['image']['tmp_name'][$key], $save_name)) continue;
            $image = new ImageSchema;
            $image->filename = $save_name;
            $image->thumbnail = self::createThubnail2($save_name);
            array_push($results, $image);
        }

        return $results;
    }

    /**
     * need [sudo apt install imagemagick]
     */
    private static function createThubnail($save_name, $thumb_name)
    {
        $command = IMAGICK . " {$save_name} -thumbnail 100x100 {$thumb_name}";
        exec($command, $out, $code);
        return $code === 0 ? $thumb_name : '';
    }

    /**
     * need [sudo apt install imagemagick]
     */
    private static function createThubnail2($save_name)
    {
        $fname = pathinfo($save_name, PATHINFO_FILENAME);
        $ext = pathinfo($save_name, PATHINFO_EXTENSION);
        $thumb_name = self::getUniqueFname("uploads/thumbnail/{$fname}.{$ext}") ?? '';

        if (empty($thumb_name)) return $save_name;

        $command = IMAGICK . " {$save_name} -thumbnail 100x100 {$thumb_name}";
        exec($command, $out, $code);
        return $code === 0 ? $thumb_name : $save_name;
    }
}

define('IMAGICK', IS_WINDOWS ? 'magick' : 'convert');
