<?php

declare(strict_types=1);

enum HttpMethod: int
{
    case UNKNOWN = 0;
    case GET = 1;
    case POST = 2;
    case PUT = 3;
    case DELETE = 4;

    public static function tryParse($text): HttpMethod
    {
        switch ($text) {
            case HttpMethod::GET->name:
                return HttpMethod::GET;
            case HttpMethod::POST->name:
                return HttpMethod::POST;
            case HttpMethod::PUT->name:
                return HttpMethod::PUT;
            case HttpMethod::DELETE->name:
                return HttpMethod::DELETE;
            default:
                return HttpMethod::UNKNOWN;
        }
    }
}

function get_param($key, $default_val, $is_post = true)
{
    $arry = $is_post ? $_POST : $_GET;
    return $arry[$key] ?? $default_val;
}

function get_item(mixed $array, string $key): string|null
{
    $value =  (is_array($array) && array_key_exists($key, $array)) ? $array[$key] : '';
    return htmlspecialchars($value);
}

function redirect($path)
{
    if ($path === GO_HOME) {
        $path = get_url('');
    } else if ($path === GO_REFERER) {
        $path = $_SERVER['HTTP_REFERER'];
    } else {
        $path = get_url($path);
    }

    header("Location: {$path}");

    die();
}

function get_url($path)
{
    return BASE_CONTEXT_PATH . trim($path, '/');
}

function get_method(): array
{
    $req_method = strtoupper($_SERVER['REQUEST_METHOD']);
    $url = parse_url(CURRENT_URI);
    $path = rtrim(str_replace(BASE_CONTEXT_PATH, '', $url['path']), '/');
    $query = $url['query'] ?? '';

    define('CURRENT_PATH', $path);

    $match = [];
    $f = fn($a) => preg_match('/method=(put|delete)$/i', $a, $match) ? strtoupper($match[1]) : '';
    $items = array_reverse(explode('?', $query));
    $items = array_values(array_diff(array_map($f, $items), ['']));

    if (is_array($items) && count($items) > 0) {
        $method = HttpMethod::tryParse($items[0]);
    } else {
        $method = HttpMethod::tryParse($req_method);
    }

    return compact('method', 'path', 'query');
}

function forwardGeocode($location)
{
    if (empty(MAPBOX_TOKEN)) return '';


    $mapbox_api = 'https://api.mapbox.com/search/geocode/v6/forward';
    $params = array(
        'q' => $location,
        'proximity' => 'ip',
        'access_token' => MAPBOX_TOKEN
    );
    $query = http_build_query($params);
    $url = "{$mapbox_api}?{$query}";

    if (IS_WINDOWS)
        $response = forwardGeocodeWin($url);
    else
        $response = forwardGeocodeLinux($url);

    if (empty($response)) return null;
    $result = json_decode($response, true);
    if (empty($result) || !is_array($result['features']) || count($result['features']) === 0) return '';
    $geometry = json_encode($result['features'][0]['geometry'], JSON_UNESCAPED_UNICODE);

    return $geometry;
}

function forwardGeocodeLinux($url)
{
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, $url);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    $response = curl_exec($ch);
    curl_close($ch);

    return $response;
}

function forwardGeocodeWin($url)
{
    $context = stream_context_create(['http' => ['timeout' => 5]]);
    $response = @file_get_contents($url, FALSE, $context);

    return $response;
}

function getGeoDatas(array $campgrounds)
{
    if (empty($campgrounds)) return '';
    $datas = array_map(fn($a) => $a->getGeoData(), $campgrounds);
    return $datas;
}
